import c4d
from c4d import gui
from c4d import utils
from c4d import storage
import random


def main():
    #ダイアログを開いてcsvファイルを選択
    csv_path = storage.LoadDialog(title="Select CSV file")
    if not csv_path:return

    #ファイルを開いてデータの読み込み
    csv = open(csv_path,'r') 
    text = csv.read()
    vl = text.split(",")
    csv.close()


    pslice = []    #入力の値の数値に変換したリスト用変数
    pslice_start = []    #円グラフ開始角度変換用変数
    pslice_end = []    #円グラフ終了角度変換用変数
    pslice_par = []    #入力値のパーセントに変換用変数

    #リスト化したCSVのデータを文字列から数値に変換
    for i in xrange(0,len(vl)):
        pslice.append(float(vl[i]))
    
    #リストの総合計
    list_total = sum(pslice)
    
    #データをソートするか確認のダイアログ
    rvalue = gui.QuestionDialog("データをソートしますか?")
    
    #イエスの場合ソートを実行
    if rvalue == True:
        pslice.sort(reverse=True)
    
    #ヌルオブジェクトの作成
    null = c4d.BaseObject(c4d.Onull)
    null[c4d.ID_BASELIST_NAME] = "Pie Chart"
    

    #各円グラフ要素を円柱で作成
    for x in xrange(0,len(vl)):
        #リストの値を割合に変換
        pslice_par.append(pslice[x]/list_total)
        #割合から円グラフの開始角度に変換さらにその角度の値をラジアン角に変換。ただし、スライスの角度が反時計回りなので-1を掛けている
        pslice_start.append(utils.Rad(360*sum(pslice[0:x])/list_total)*-1)
        
        #割合から円グラフの終了角度に変換さらにその角度の値をラジアン角に変換。ただし、スライスの角度が反時計回りなので-1を掛けている
        pslice_end.append(utils.Rad(360*sum(pslice[0:x+1])/list_total)*-1)
        
        #円グラフの円柱の全体の分割数を80として、それに合わせた各要素の分割数を判定
        segment = 80*pslice_par[x]

        #円グラフ用の円柱のパラメータを設定
        cylinder = c4d.BaseObject(c4d.Ocylinder)    #円柱の作成
        cylinder[c4d.PRIM_SLICE] = True    #円柱のスライスを有効にする
        cylinder[c4d.PRIM_SLICEFROM] = pslice_start[x]    #スライスのからに入力
        cylinder[c4d.PRIM_SLICETO] = pslice_end[x]    #スライスのまでに入力
        cylinder[c4d.ID_BASEOBJECT_REL_ROTATION,c4d.VECTOR_X] = c4d.utils.Rad(90)    #円柱の0°が3時の位置からスタートしているので90°回転
        cylinder[c4d.PRIM_CYLINDER_SEG] = int(segment)    #円柱の分割数を入力
        cylinder[c4d.PRIM_CYLINDER_HEIGHT] = 10    #円柱の高さを指定
        cylinder[c4d.PRIM_CYLINDER_FILLET] = True    #円柱のフィレットを有効にする
        cylinder[c4d.PRIM_CYLINDER_FILLETRADIUS] = 1    #円柱のフィレットの半径を入力
        cylinder[c4d.ID_BASELIST_NAME] = str(int(pslice_par[x]*100)) + "%"    #円柱の名前を割合に設定
        cylinder[c4d.ID_BASEOBJECT_USECOLOR] = True    #円柱の色を指定をオン
        cylinder[c4d.ID_BASEOBJECT_COLOR] = c4d.Vector(random.random(),random.random(),random.random())    #円柱の表示色をランダムに生成

        #スムースタグを追加
        cylinder.MakeTag(c4d.Tphong)

        #ヌルオブジェクトに円柱を挿入
        cylinder.InsertUnder(null)
    
    #ヌルをシーンに挿入
    doc.InsertObject(null)
    c4d.EventAdd()

if __name__=='__main__':
    main()
